<?php

add_action('admin_menu', 'add_menu_option');

global $cacheDeletionTimes;

$cacheDeletionTimes['hourly'] = array('label' => 'One hour',
									  'sql' => '1 HOUR');
$cacheDeletionTimes['dayly'] = array('label' => 'One day',
									  'sql' => '1 DAY');
$cacheDeletionTimes['weekly'] = array('label' => 'One week',
									  'sql' => '1 WEEK');
$cacheDeletionTimes['fortnightly'] = array('label' => 'One fortnight',
									  'sql' => '2 WEEK');
$cacheDeletionTimes['monthly'] = array('label' => 'One month',
									  'sql' => '1 MONTH');

function cacheCombo($default)
{
	global $cacheDeletionTimes;
	$i = -1;
	$options = array();
	
	echo '<select name="cachelength" id="cachelength">';

	foreach ($cacheDeletionTimes as $key => $value) {
		$i++;

		if ($default == $key) {
			$options[$i]['default'] = 1;
		}
		
		$options[$i]['value'] = $key;
		$options[$i]['label'] = $value['label'];
	}
	
	echo comboOptions($options);
	
	echo '</select>';
}

function filesCombo($default)
{
	$allFiles = array();
	$options = array();
	$datCount = -1;
	
	$allFiles = scandir(WP_PLUGIN_DIR . '/regiondetect/');
	
	// iterate through the files/folders
	for ($i = 0 ; $i <= sizeof($allFiles) - 1 ; $i++) {
		// is this a .dat file ?
		if (strpos($allFiles[$i], '.dat') !== false) {
			$datCount++;
			
			$options[$datCount]['label'] = $allFiles[$i];
			
			// is this the default ?
			if ($default == $allFiles[$i]) {
				$options[$datCount]['default'] = 1;
			}
		}
	}
	
	if ($datCount != -1) {
		echo '<select name="dbfile" id="dbfile">';
		
		echo comboOptions($options);
	} else {
		echo '<select disabled="disabled" name="dbfile" id="dbfile">';
		
		echo '<option value="">No Files Available</option>';
	}
	
	echo '</select>';
}

function comboOptions($options)
{
	/* The $options array should contain an array for each option
	 * that contains the following elements:
	 *
	 * default - optional
	 * value - optional
	 * label - mandatory
	 */
	 $optionsString = '';
	 
	 foreach ($options as $option) {
		$optionsString .= '<option' .
		
		// default?
		(isset($option['default']) ? ' selected' : '') .
		
		// do we have a value?
		(isset($option['value']) ? ' value="' . $option['value'] . '"' : '') .
		
		// end of the opening tag
		'>' .
		
		// what actually gets displayed in the cmbo box
		$option['label'] .
		
		// closing tag
		'</option>';
	 }

	 return $optionsString;
}
									  
function add_menu_option()
{
	add_submenu_page('options-general.php', 'Region Detect Options', 'Region Detect', 8, 'regiondetect', 'regiondetect_options');
}

function regiondetect_options()
{
	global $wpdb, $table_prefix;
	$message = '';
	$errors = array();
	
	if (isset($_POST['licensekey'])) {
		update_option('rd_licensekey', $_POST['licensekey']);
		update_option('rd_source', (int)$_POST['source']);
		update_option('rd_usecaching', (isset($_POST['usecaching'])) ? 1 : 0);
		update_option('rd_urloverride', (isset($_POST['urloverride'])) ? 1 : 0);
		update_option('rd_urlparameter', $_POST['urlparameter']);
		update_option('rd_cachelength', $_POST['cachelength']);
		update_option('rd_dbfile', $_POST['dbfile']);
		update_option('rd_defaultcountrycode', $_POST['defaultcountrycode']);
		
		// should we remove entries in the cache?
		if (!isset($_POST['usecaching'])) {
			$wpdb->query("DELETE FROM {$table_prefix}rd_ipcache");
		}
		
		$message = 'Region detect options updated.';
	}
	
	if (isset($_POST['searchenginerules'])) {
		update_option('rd_botrules', $_POST['searchenginerules']);
		update_option('rd_checkbotrules', (isset($_POST['checkbotrules'])) ? 1 : 0);
		
		if (!isset($_POST['checkbotrules'])) {
			// remove all entries from the user agent log
			$wpdb->query("DELETE FROM {$table_prefix}rd_useragentlog");
		}
		
		$message = 'Search engine exclusion rules updated.';
	}
	
	$source = get_option('rd_source');
	
?>
<div class="wrap">
  <form action="<?php echo get_option('siteurl') . '/wp-admin/admin.php?page=regiondetect'; ?>" method="post">
	<h2>Region Detect</h2>
<?php

	if ($message) {
		echo '<div class="message updated" style="width: 97%;" id="message"><p><strong>' . $message . '</strong></p></div>';
	}
	
	if (sizeof($errors)) {
		echo '<div class="message error" style="width: 97%;" id="message"><p><strong>' . implode('<br />', $errors) . '</strong></p></div>';
	}

?>
	<div class="postbox-container">
	<div class="metabox-holder">

	<div style="margin-right:100%;" class="postbox">
	<h3 class="hndle">Options</h3>
	<table width="100%" style="margin:8px;" class="form-table">
<tr valign="top">
	<td style="text-align:left;"><input type="text" name="defaultcountrycode" size="2" maxlength="2" value="<?php echo get_option('rd_defaultcountrycode'); ?>" /></td>
	<td><strong>Default country code</strong><br />
	This country code will be used if the plugin cannot determin the country. This must be the two character <a target="_blank" href="http://en.wikipedia.org/wiki/ISO_3166-1">ISO 3166-1</a> code</td>
</tr>
<tr valign="top">
	<th scope="row" style="text-align:left;"><input type="text" name="urlparameter" size="15" value="<?php echo get_option('rd_urlparameter'); ?>" /></th>
	<td><strong>URL Parameter</strong><br />
	This is the url parameter key.</td>
</tr>
<tr valign="top">
	<th scope="row" style="text-align:left;">
	<input type="checkbox" name="urloverride" id="urloverride" <?php echo (get_option('rd_urloverride') ? 'checked="checked"' : ''); ?> />
	</th>
	<td><strong>Override IP service with url parameter</strong><br />
	This option forces the plugin to check the url parameter before the selected service below. If caching is switched on, this option will overwrite the IP in the cache if it exists.</td>
</tr>
<tr valign="top">
	<th scope="row" style="text-align:left;">
	<input type="radio" name="source" id="urlparam" <?php echo (($source == RD_URLPARAM) ? 'checked="checked"' : ''); ?> value="<?php echo RD_URLPARAM; ?>" /> <label for="urlparam">URL Parameter</label><br />
	<input type="radio" name="source" id="maxmindwebservice" <?php echo (($source == RD_MAXMINDWEB) ? 'checked="checked"' : ''); ?> value="<?php echo RD_MAXMINDWEB; ?>" /> <label for="maxmindwebservice">Maxmind Web</label><br />
	<input type="radio" name="source" id="maxmindfile" <?php echo (($source == RD_MAXMINDFILE) ? 'checked="checked"' : ''); ?> value="<?php echo RD_MAXMINDFILE; ?>" /> <label for="maxmindfile">Maxmind File</label>
	</th>
	<td><strong>IP Address to Country service</strong><br />
	This plugin currently supports database lookups from products from MaxMind. Please use my <a target="_blank" href="http://www.brewsterware.com/maxmind.php">MaxMind Affiliate link</a> so that I can continue to produce plugins and provide support for free. This plugin also works with the free MaxMind database.</td>
</tr>
<tr valign="top">
	<th scope="row" style="text-align:left;"><input type="text" name="licensekey" size="15" value="<?php echo get_option('rd_licensekey'); ?>" /></th>
	<td><strong>License Key</strong><br />
	This license key will be used if the service you have selected requires a license key.</td>
</tr>
<tr valign="top">
	<th scope="row" style="text-align:left;">
	<?php filesCombo(get_option('rd_dbfile')); ?>
	</th>
	<td><strong>Database File</strong><br />
	This is a list of database files that are in the plugin folder. This file will be used if the service selected requires a database file.</td>
</tr>
<tr valign="top">
	<th scope="row" style="text-align:left;">
	<input type="checkbox" name="usecaching" id="usecaching" <?php echo (get_option('rd_usecaching') ? 'checked="checked"' : ''); ?> />
	</th>
	<td><strong>Cache ip addresses</strong><br />This option determins whether the ip addresses are cached - this is recommended if you are using a service where you are paying for each ip address lookup or if you are using the URL parameter to get the country code. Enabling this option will also add a table below showing the contents of the cache. Disabling this option and saving will remove all entries in the cache.</td>
</tr>
<tr valign="top">
	<th scope="row" style="text-align:left;">
	<?php cacheCombo(get_option('rd_cachelength')); ?>
	</th>
	<td><strong>Cache length</strong><br />This option determins the length of time that the ip address is cached for.</td>
</tr>
<tr valign="top">
	<th scope="row" style="text-align:left;">
	</th>
	<td><p class="submit"><input class="button-primary" type="submit" name="go" value="Save Settings"></p></td>
</tr>
	</table>
	</form>
	</div>
	
<?php
	if (get_option('rd_usecaching')) {
?>
	<table class="widefat">
		<thead>
		<tr>
			<th>Flag</th>
			<th>Country</th>
			<th>IP Addresses</th>
		</tr>
		</thead>
		<tfoot>
		<tr>
			<th>Flag</th>
			<th>Country</th>
			<th>IP Addresses</th>
		</tr>
		</tfoot>

<?php
	$query = "	SELECT
						COUNT(ipcache.ip)
					AS	ipcount,
						LOWER(ipcache.country)
					AS	country,
						countries.countryName
				FROM
						{$table_prefix}rd_ipcache
					AS	ipcache
				INNER JOIN
						{$table_prefix}rd_countrynames
					AS	countries
					ON	(countries.countrycode = ipcache.country)
				GROUP BY
						ipcache.country
				ORDER BY
						COUNT(ipcache.ip) DESC";

	$rows = $wpdb->get_results($query);
	
	$iconFolder = get_option('siteurl') . '/wp-content/plugins/regiondetect/images/icons/';
	
	if (sizeof($rows)) {
		echo "<tbody>\n";
		
		foreach ($rows as $row) {
			echo "<tr>\n";
			echo "<td><img src=\"$iconFolder" . $row->country . ".png\" /></td>\n";
			echo "<td>{$row->countryName}</td>\n";
			echo "<td>{$row->ipcount}</td>\n";
			echo "</tr>\n";
		}
		
		echo "</tbody>\n";
	}
	
	echo '</table><br />';
	
	}

?>
	
	<form action="<?php echo get_option('siteurl') . '/wp-admin/admin.php?page=regiondetect'; ?>" method="post">
	<div style="margin-right:100%;" class="postbox">
	<h3 class="hndle">Search Engine Robot Exclusion Rules</h3>
	<table width="100%" style="margin:8px;" class="form-table">
	<tr>
	<td colspan="2">These rules work by looking for a match anywhere within the useragent. If a match is found, no country lookup is performed. These rules are case insensitive.<br /><br />
	<textarea name="searchenginerules" rows="8" cols="100"><?php echo get_option('rd_botrules'); ?></textarea></td>
	</tr>
	<tr valign="top">
	<th scope="row" style="text-align:left;"><input type="checkbox" name="checkbotrules" id="checkbotrules" <?php echo ((int)get_option('rd_checkbotrules') == 1) ? 'checked="checked"' : ''; ?>></th>
	<td><label for="checkbotrules"><strong>Check search engine bot rules</strong></label><br />
	Turning this option on will log the hits and exclusions made by all user agents. Results are shown in the table below.
	</td>
	</tr>
	<tr>
	<td colspan="2"><p class="submit"><input class="button-primary" type="submit" name="go" value="Save Settings"></p></td>
	</tr>
	</table>
	</div>
	
	<table class="widefat">
		<thead>
		<tr>
			<th>User Agent</th>
			<th>Hits</th>
			<th>Exclusions</th>
		</tr>
		</thead>
		<tfoot>
		<tr>
			<th>User Agent</th>
			<th>Hits</th>
			<th>Exclusions</th>
		</tr>
		</tfoot>
<?php
	
	$query = "	SELECT
						(CASE
							WHEN length(useragent) = 0
							THEN 'No User Agent Specified'
							ELSE useragent
						END)
					AS	useragent,
						hits,
						exclusions
				FROM
						{$table_prefix}rd_useragentlog
				ORDER BY
						hits DESC,
						exclusions DESC";

	$rows = $wpdb->get_results($query);
	
	if (sizeof($rows)) {
		echo "<tbody>\n";
		
		foreach ($rows as $row) {
			echo "<tr>\n";
			echo "<td>{$row->useragent}</td>\n";
			echo "<td>{$row->hits}</td>\n";
			echo "<td>{$row->exclusions}</td>\n";
			echo "</tr>\n";
		}
		
		echo "</tbody>\n";
	}
	
?>
	</table>
	
	</form>
	
	</div>
	</div>
	</div>
<?php
}


?>