<?php

/*
Plugin Name: Region Detect
Plugin URI: http://www.brewsterware.com/region-detect-wordpress-plugin.html
Version: 1.02.02
Description: Provides country code information to other plugins. Use %%COUNTRYCODE%% and %%COUNTRYNAME%% within posts to provide the country code and country name respectively.
Author: Joe Brewer
Author URI:  http://www.brewsterware.com/
*/

/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

require_once('interface.php');

define('RD_COUNTRYNAME', '%%COUNTRYNAME%%');
define('RD_COUNTRYCODE', '%%COUNTRYCODE%%');

define('RD_MAXMINDFILE', 0);
define('RD_MAXMINDWEB', 1);
define('RD_URLPARAM', 2);

global $regiondetect_db_version,
		$rdCountryName,
		$rdISO3166_1;

$regiondetect_db_version = 1;

register_activation_hook(__FILE__, 'install_regiondetect');
register_uninstall_hook(__FILE__, 'uninstall_regiondetect');

add_action('init', 'regiondetect_main');
add_filter('the_content', 'checkRegionTokens', 250);

function checkRegionTokens($PostBody)
{
	global $rdISO3166_1, $rdCountryName;
	
#	if (!is_user_logged_in()) {
		// country name
		if (strpos($PostBody, RD_COUNTRYNAME) !== false) {
			$PostBody = str_replace(RD_COUNTRYNAME,
									$rdCountryName,
									$PostBody);
		}
		
		// country code
		if (strpos($PostBody, RD_COUNTRYCODE) !== false) {
			$PostBody = str_replace(RD_COUNTRYCODE,
									$rdISO3166_1,
									$PostBody);
		}
#	}
	
	return $PostBody;
}

function install_regiondetect()
{
	global $wpdb, $table_prefix;
	
	$tables["{$table_prefix}rd_ipcache"] = "CREATE TABLE `{$table_prefix}rd_ipcache` (
			`ip` int(11) NOT NULL,
			`country` varchar(2) NOT NULL,
			`dateAdded` datetime NOT NULL,
			PRIMARY KEY  (`ip`),
			KEY `dateAdded` (`dateAdded`)
			) ENGINE=MyISAM DEFAULT CHARSET=utf8;";
	
	$tables["{$table_prefix}rd_useragentlog"] = "CREATE TABLE `{$table_prefix}rd_useragentlog` (
			`useragent` VARCHAR(255) NOT NULL,
			`hits` INT NOT NULL,
			`exclusions` INT NOT NULL
			) ENGINE=MYISAM;";
	
	$tables["{$table_prefix}rd_countrynames"] = "CREATE TABLE `{$table_prefix}rd_countrynames` (
			`countryCode` VARCHAR(2) character set utf8 NOT NULL,
			`countryName` VARCHAR(64) character set utf8 NOT NULL,
			PRIMARY KEY (`countryCode`)
			) ENGINE=MYISAM;";
	
	
	// create tables if they do not already exist
	foreach ($tables as $tableName => $tableDesign) {
		if ($wpdb->get_var("show tables like '$tableName'") != $tableName) {
			$wpdb->query($tableDesign);
		}
	}
	
	// get the current database version
	$currentDbVersion = (int)get_option('regiondetect_db_version');
		
	// do we need to upgrade the database?
	if ($regiondetect_db_version > $currentDbVersion) {
		// run through the upgrades version by version
		for ($version = $currentDbVersion ;
			 $version <= $regiondetect_db_version ;
			 $version++) {
			// is this an array ?
			if (is_array($dbChanges[$version])) {
				// iterate through the changes for each version
				foreach ($dbChanges[$version] as $change) {
					if ($change) {
						$wpdb->query($change);
					}
				}
			}
		}
	
		// update the database version option
		update_option('regiondetect_db_version', $countrydetect_db_version);
	}
	
	// fill the country names table
	$countryCodes = @file(WP_PLUGIN_DIR . '/regiondetect/countries.txt');
	if (is_array($countryCodes) &&
		sizeof($countryCodes)) {
		foreach ($countryCodes as $country) {
			// a comment? move along please..
			if (substr($country, 0, 1) == '#') {
				continue;
			}
			
			// check that we have data in the format that we expect
			if ($country &&
				strpos($country, ',') !== false) {
				// initialise variables
				$code = '';
				$name = '';
				
				// extract the code and name
				list($code, $name) = explode(',', $country);
				
				// insert or replace the record
				$query = "	REPLACE INTO {$table_prefix}rd_countrynames
								(countryCode, countryName)
							VALUES
								(%s, %s)";
				$wpdb->query($wpdb->prepare($query, trim($code), trim($name)));
			}
		}
	}
	
	// add the options
	add_option('rd_webquery_counter', 0);
	add_option('rd_bot_counter', 0);
	add_option('rd_botrules', '');
	add_option('rd_checkbotrules', 0);
	add_option('rd_licensekey', '');
	add_option('rd_dbfile', '');
	add_option('rd_source', 0);
	add_option('rd_usecaching', 0);
	add_option('rd_cachelength', 'hourly');
	add_option('rd_defaultcountrycode', 'US');
	add_option('rd_urlparameter', 'country');
	add_option('rd_urloverride', 0);
	
	// load the robot rules if the file exists
	if (($rules = @file(WP_PLUGIN_DIR . '/regiondetect/rules.txt')) != false) {
		// clean up the array
		for ($i = 0 ; $i <= sizeof($rules)-1 ; $i++) {
			$rules[$i] = trim($rules[$i]);
		}
		
		// turn the array into a string
		$rulesString = implode("\r", $rules);
		
		// load the string into the option
		update_option('rd_botrules', $rulesString);
	}
}


function uninstall_regiondetect()
{
	global $wpdb, $table_prefix;
	
	// remove tables from the database
	$wpdb->query("DROP table {$table_prefix}rd_ipcache");
	$wpdb->query("DROP table {$table_prefix}rd_useragentlog");
	$wpdb->query("DROP table {$table_prefix}rd_countrynames");

	// remove options
	delete_option('regiondetect_db_version');
	delete_option('rd_webquery_counter');
	delete_option('rd_bot_counter');
	delete_option('rd_botrules');
	delete_option('rd_checkbotrules');
	delete_option('rd_licensekey');
	delete_option('rd_dbfile');
	delete_option('rd_source');
	delete_option('rd_usecaching');
	delete_option('rd_cachelength');
	delete_option('rd_defaultcountrycode');
	delete_option('rd_urlparameter');
	delete_option('rd_urloverride');
}

function regiondetect_main()
{
	global $wpdb, $table_prefix, $rdISO3166_1, $rdCountryName;
	$regionDetect = new regiondetect();

	// are we debugging?
	if (get_option('rd_checkbotrules')) {
		$useragent = $_SERVER['HTTP_USER_AGENT'];
		
		// check to see if we already have the useragent in the db
		$query = "	SELECT
							COUNT(useragent)
					FROM
							{$table_prefix}rd_useragentlog
					WHERE
							useragent = %s;";
		$useragentCount = $wpdb->get_var($wpdb->prepare($query, $useragent));
		
		if (!$useragentCount) {
			// no user agent found - prepare the data
			$data = array('useragent' => $useragent,
							'hits' => '1',
							'exclusions' => '0');
			
			// insert the data
			$wpdb->insert("{$table_prefix}rd_useragentlog",
							$data,
							array('%s', '%d', '%d'));
		} else {
			// user agent found - just update the number of hits
			$query = "	UPDATE
								{$table_prefix}rd_useragentlog
						SET
								hits = hits + 1
						WHERE
								useragent = %s";
			$wpdb->query($wpdb->prepare($query, $useragent));
		}
	}
	
	$regionDetect->defaultCountry = get_option('rd_defaultcountrycode');
	
	// set our license key
	$regionDetect->licenceKey = get_option('rd_licensekey');
	
	// set the filename
	$regionDetect->fileName = WP_PLUGIN_DIR . '/regiondetect/' . get_option('rd_dbfile');

	// should we use caching?
	$regionDetect->useCaching = get_option('rd_usecaching');
	
	// url stuff
	$regionDetect->urlOverride = (get_option('rd_urloverride') == 1);
	$regionDetect->urlParam = get_option('rd_urlparameter');
	
	// are we using the web service
	$regionDetect->source = get_option('rd_source');
	
	// lookup the country
	$regionDetect->lookupCountry();
	
	// set the global country code variable
	$rdISO3166_1 = $regionDetect->iso;
	
	// lookup the country name
	$query = "	SELECT
						countryName
				FROM
						{$table_prefix}rd_countrynames
				WHERE
						countryCode = %s";
	$preparedQuery = $wpdb->prepare($query, $rdISO3166_1);
	$rdCountryName = $wpdb->get_var($preparedQuery);
	
	// did we use the web service ?
	if ($regionDetect->usedWebService) {
		// increment our counter...
		update_option('rd_webquery_counter',
					(int)get_option('rd_webquery_counter') + 1);
	}
	
	// did we successfully detect a bot ?
	if ($regionDetect->isBot) {
		// increment out bot counter..
		update_option('rd_bot_counter',
					(int)get_option('rd_bot_counter') + 1);
		
		// are we debugging?
		if (get_option('rd_checkbotrules')) {
			$useragent = $_SERVER['HTTP_USER_AGENT'];
			
			$query = "	UPDATE
								{$table_prefix}rd_useragentlog
						SET
								exclusions = exclusions + 1
						WHERE
								useragent = %s";
			$wpdb->query($wpdb->prepare($query, $useragent));
		}
	}
}

class regiondetect {
	var $source = -1;
	
	// stuff for the web service
	var $licenseKey = "";
	var $ip = "";
	var $usedWebService = false;
	var $urlParam = 'country';
	var $urlOverride = false;
	
	var $useCaching = false;
	var $robotExclusionRules = array();
	
	// stuff for the file
	var $fileName = "";
	
	// everything else
	var $defaultCountry = "US";
	var $iso = "";
	var $isBot = false;
	
	
	// Constructor
	function regiondetect()
	{
		// Get the IP address
		$this->ip = $this->GetIP();
		
		// are we a bot ?
		$this->isBot = $this->checkUserAgent();
		
		// expire the cache
		$this->expireCache();
	}
	
	function lookupCountry()
	{
		// have we found the country ?
		if (!$this->iso) {
			// over ride using the url parameter?
			if ($this->urlOverride) {
				if ($this->getCountryFromURL(false)) {
					/* we got something :-)
					 * do we need to cache it?
					 */
					if ($this->useCaching) {
						/* update the cached record if it exists
						 * or create it if it doesnt
						 */
						$this->cacheIp($this->ip, $this->iso, true);
					}
					
					// no need to do anything else
					return;
				}
			}
		
			// are we caching ip addresses?
			if ($this->useCaching) {
				// check the cache
				$this->checkCache($this->ip);
				
				// did we find it?
				if ($this->iso) {
					return;
				}
			}
			
			// get the country code
			switch ($this->source) {
				case RD_URLPARAM:
					$this->getCountryFromURL();
					break;
					
				case RD_MAXMINDWEB:
					$this->getCountryFromMaxmindWeb();
					break;
				
				case RD_MAXMINDFILE:
					$this->getCountryFromMaxmindFile();
					break;
				
				default:
					$this->iso = $this->defaultCountry;
					
					// dont really want to cache the ip address
					return;
			}
			
			// have we got an ip address, or did something go wrong somewhere?
			if (!$this->iso) {
				$this->iso = $this->defaultCountry;
				
				// dont bother caching as we're using the default
				return;
			}
		
			// cache the ip
			if ($this->useCaching) {
				$this->cacheIp($this->ip, $this->iso);
			}
		}
	}
	
	private function cacheIP($ip, $countryCode, $update = false)
	{
		global $wpdb, $table_prefix;
		
		// prepare the data
		$data = array('ip' => ip2long($ip),
						'country' => $countryCode);
		
		$command = ($update) ? 'REPLACE' : 'INSERT';
		
		$query = "	$command INTO {$table_prefix}rd_ipcache
						(ip, country, dateAdded)
					VALUES
						(%d, %s, NOW())";
		$preparedQuery = $wpdb->prepare($query,
										$data['ip'],
										$data['country']);
		$wpdb->query($preparedQuery);
	}
	
	private function checkCache($ip)
	{
		global $wpdb, $table_prefix;
		
		$longIp = ip2long($ip);
		$query = "	SELECT
							country
					FROM
							{$table_prefix}rd_ipcache
					WHERE
							ip = $longIp";
		
		$row = $wpdb->get_row($query, ARRAY_N);
		
		if (is_array($row)) {
			$this->iso = $row[0];
		}
	}
	
	private function expireCache()
	{
		global $wpdb, $table_prefix, $cacheDeletionTimes;
		
		// get the sql snippet
		$expirationLength = $cacheDeletionTimes[get_option('rd_cachelength')]['sql'];
		
		// do we need to do anything?
		if (!get_option('rd_usecaching') ||
			!$expirationLength) {
			return;
		}
 		
		$query = "	DELETE FROM {$table_prefix}rd_ipcache
					WHERE dateAdded < DATE_SUB(NOW(), INTERVAL $expirationLength)";
		$wpdb->query($query);
	}


	function getCountryFromURL($useDefault = true)
	{
		// is anything there?
		if (!isset($_GET[$this->urlParam]) ||
			strlen($_GET[$this->urlParam]) <> 2) {
			// nope :-(
			
			// should we use the default ?
			if ($useDefault) {
				$this->iso = $this->defaultCountry;
			}
			
			// dont do anything more
			return false;
		}
		
		// we found something :-)
		$this->iso = $_GET[$this->urlParam];
		
		return true;
	}
	
	
	function getCountryFromMaxmindFile()
	{
		require_once("geoip.php");
		
		// does the database exist?
		if (!$this->fileName ||
			!@file_exists($this->fileName)) {
			$this->iso = $this->defaultCountry;

			return;
		}

		if (($gi = geoip_open($this->fileName, GEOIP_STANDARD)) != false) {
			$this->iso = geoip_country_code_by_addr($gi, $this->ip);
		} else {
			$this->iso = $this->defaultCountry;
		}
	}
	
	function getCountryFromMaxmindWeb()
	{
		/* if this is a bot, or we already know the country, set the
		 * default and abort....
		 */
		if ($this->isBot) {
			$this->iso = $this->defaultCountry;
			return;
		}
		
		// do we already know what the country is ?
		if (strlen($this->iso)) {
			return;
		}
		
		$this->usedWebService = true;
		
		$query = "http://geoip3.maxmind.com/a?l=" . $this->licenseKey . "&i=" . $this->ip;
		
		$url = parse_url($query);
		$host = $url["host"];
		$path = $url["path"] . "?" . $url["query"];
		$timeout = 1;
		
		// open a connection to the web service
		$fp = fsockopen($host, 80, $errno, $errstr, $timeout);
			
		// did it open ok ?
		if ($fp) {
			// request the ip address
			fputs($fp, "GET $path HTTP/1.0\nHost: " . $host . "\n\n");
			
			$buf = '';
			
			// read the response
			while (!feof($fp)) {
				$buf .= fgets($fp, 128);
			}
			
			// close the connection
			fclose($fp);
			
			// extract the country
			$lines = split("\n", $buf);
			$this->iso = trim($lines[count($lines) - 1]);
			
			// check the response
			if (strlen($this->iso) <> 2) {
				/* the response from the web service was not valid
				 * so use the default
				 */
				$this->iso = $this->defaultCountry;
			}
		} else {
			// use the default country
			$this->iso = $this->defaultCountry;
		}
	}


	function ValidIP($ip)
	{
		if (!empty($ip) && ip2long($ip) != -1) {
			$reserved_ips = array (
				array('0.0.0.0','2.255.255.255'),
				array('10.0.0.0','10.255.255.255'),
				array('127.0.0.0','127.255.255.255'),
				array('169.254.0.0','169.254.255.255'),
				array('172.16.0.0','172.31.255.255'),
				array('192.0.2.0','192.0.2.255'),
				array('192.168.0.0','192.168.255.255'),
				array('255.255.255.0','255.255.255.255')
			);

			foreach ($reserved_ips as $r) {
				$min = ip2long($r[0]);
				$max = ip2long($r[1]);
				if ((ip2long($ip) >= $min) && (ip2long($ip) <= $max)) return false;
			}

			return true;
		} else {
			return false;
		}
	}

	function GetIP()
	{
		if ($this->ValidIP($_SERVER["HTTP_CLIENT_IP"])) {
			return $_SERVER["HTTP_CLIENT_IP"];
		}

		foreach (explode(",",$_SERVER["HTTP_X_FORWARDED_FOR"]) as $ip) {
			if ($this->ValidIP(trim($ip))) {
				return $ip;
			}
		}

		if ($this->ValidIP($_SERVER["HTTP_X_FORWARDED"])) {
			return $_SERVER["HTTP_X_FORWARDED"];
		} elseif ($this->ValidIP($_SERVER["HTTP_FORWARDED_FOR"])) {
			return $_SERVER["HTTP_FORWARDED_FOR"];
		} elseif ($this->ValidIP($_SERVER["HTTP_FORWARDED"])) {
			return $_SERVER["HTTP_FORWARDED"];
		} elseif ($this->ValidIP($_SERVER["HTTP_X_FORWARDED"])) {
			return $_SERVER["HTTP_X_FORWARDED"];
		} else {
			return $_SERVER["REMOTE_ADDR"];
		}
	}
	
	private function checkUserAgent()
	{
		$bots = array();
		
		// block blank useragents as well
		if (!strlen($_SERVER['HTTP_USER_AGENT'])) {
			return true;
		}
		
		$bots = explode("\n", get_option('rd_botrules'));
		
		// was there anything in the file ?
		if (is_array($bots) &&
			sizeof($bots)) {
			/* check each bot in the array by checking for
			 * a partial match on the user agent string.
			 */
			foreach ($bots as $bot) {
				if (strlen(trim($bot)) &&
					stripos($_SERVER['HTTP_USER_AGENT'], trim($bot)) !== false) {
					return true;
				}
			}
		}
		
		// either this isnt a bot, or there was nothing in the file
		return false;
	}

}

?>